#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Vhffs::Panel::Group;

use DBI;
use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use strict;
use Vhffs::User;
use Vhffs::Main;
use Vhffs::Panel::Main;


sub getall_users_per_group
{
	my $main = shift;
	my $groupname = shift;

	my $query = "SELECT u.username, g.groupname from vhffs_users u, vhffs_groups g, vhffs_user_group ug WHERE ug.gid = g.gid AND ug.uid=u.uid AND g.groupname='".$groupname."' AND ug.state='6'";
	my $request = $main->{'db'}->{'DB_READ'}->prepare( $query );

	return undef if ( $request->execute <= 0 );
	return( $request->fetchall_hashref('username') );
}

sub getall_groups
{
    my $main = shift;
	my $name = shift;

    my $query;

	if( defined $name )
	{
		$query =  "SELECT g.groupname, g.object_id, g.quota, g.quota_used, o.state FROM vhffs_groups g, vhffs_object o WHERE g.object_id=o.object_id AND g.groupname LIKE '%".$name."%'";
	}
	else
	{
		$query =  "SELECT g.groupname, g.object_id, g.quota, g.quota_used, o.state FROM vhffs_groups g, vhffs_object o WHERE g.object_id=o.object_id AND g.groupname NOT IN ( SELECT username FROM vhffs_users )";
	}
    my $request = $main->{'db'}->{'DB_READ'}->prepare( $query );

    return undef if ( $request->execute <= 0 );
    return( $request->fetchall_hashref('groupname') );

}



sub getall_groups_to_moderate
{
    my( $main , $query , $request);
    $main = shift;
	$query = "SELECT o.description ,  u.username , g.groupname , o.object_id FROM vhffs_object o , vhffs_users u , vhffs_groups g  , vhffs_groups_info gi WHERE u.uid=gi.owner_uid AND gi.gid=g.gid AND u.username!=g.groupname AND g.object_id=o.object_id AND o.state='".Vhffs::Constants::WAITING_FOR_VALIDATION."'";
    $request = $main->{'db'}->{'DB_READ'}->prepare( $query );

    return undef if ( $request->execute <= 0 );
    return( $request->fetchall_hashref('groupname') );
}


sub getall_groups_per_user
{
	my ( $user , $main ) = @_;

	return undef if ( $user->fetch < 0 );

	my $query = "SELECT g.groupname , g.object_id, o.state  FROM vhffs_groups g, vhffs_user_group ug , vhffs_object o WHERE o.object_id=g.object_id AND ug.gid=g.gid AND ug.uid='".$user->{'uid'}."'";

	my $request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -1;
	my @retour;

	return undef if ( $request->execute() <= 0);
	return ( $request->fetchall_hashref( 'groupname' ) );
}

sub create_group
{
	my( $groupname , $user , $main ) = @_;

	return undef if( $user->fetch < 0);

	my $group = new Vhffs::Group( $main , $groupname , $user->get_uid) ;

	return -1 if( ! defined $group );

	return -2 if ($group->create < 0);

	return -3 if ($group->add_user( $user ) < 0 );

    return -4 if ( Vhffs::Acl::add_acl( $user , $group , Vhffs::Constants::ACL_DELETE , $main ) < 0 ); 

	return 1;
}


sub menu
{
	my ( $class , $data ) = @_;

	my $vhffs = $data->{'vhffs'};
	my $session = $data->{'session'};
	my $user = $data->{'user'};
	my $cgi = $data->{'cgi'};
	my $output = "";
    my $templatedir = $vhffs->get_config->get_templatedir;
	my $template = new HTML::Template( filename => $templatedir."/group/menu.tmpl" );
	$template->param( TEXT_MYPROJECTS => gettext("My Projects") );
	$template->param( SHOW_HIDE => gettext("Show/Hide") );
	$template->param( TEXT_CREATE => gettext("Create") );

	# projects 

#	my $groups = getall_groups_per_user( $user , $vhffs );
	my $groups = $user->get_groups;
	my $obj;
	my $subtemplate;	
	foreach $obj ( @{$groups} )
	{
		if( $obj->get_status == Vhffs::Constants::ACTIVATED )
		{
			$subtemplate = new HTML::Template( filename => $templatedir."/group/menu_sub.tmpl" );
			$subtemplate->param( PROJECTNAME => $obj->get_groupname );
			$output .= $subtemplate->output;
		}
		else
		{
			$subtemplate = new HTML::Template( filename => $templatedir."/group/menu_sub_wait.tmpl" );
			$subtemplate->param( PROJECTNAME => $obj->get_groupname );
			$subtemplate->param( REASON => "(". Vhffs::Functions::status_string_from_status_id( $obj->get_status ) .")" );
			$output .= $subtemplate->output;
		}
	}
	

	$template->param( SUB_GROUP => $output );

	return $template->output;
}

1;
