#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Vhffs::Panel::LargeFile;

use DBI;
use POSIX qw(locale_h);
use HTML::Template;
use locale;
use Locale::gettext;
use CGI;
use CGI::Session;
use strict;
use Vhffs::User;
use Vhffs::Main;
use Vhffs::Panel::Main;
use Vhffs::Services::LargeFile;



sub menu
{
	my $class = shift;
	my $data = shift;

	my $vhffs = $data->{'vhffs'};
	my $session = $data->{'session'};
	my $user = $data->{'user'};
	my $group = $data->{'group'};
	my $cgi = $data->{'cgi'};

	my $templatesdir = $vhffs->get_config->get_templatedir;
	my $template = new HTML::Template( filename => "$templatesdir/largefile/menu.tmpl" );
	$template->param( TEXT_CREATE => gettext("Create") );
	$template->param( TEXT_LARGEFILE => gettext("Hosted files") );

	my $subtemplate;

	my $objs = Vhffs::Services::LargeFile::getall( $vhffs , undef , undef , $group );
	my $obj;
	my $output = "";
	if( defined $objs )
	{
		foreach $obj ( @{$objs} )
		{
			if( ( $obj->get_status == Vhffs::Constants::ACTIVATED ) || ( $obj->get_status == Vhffs::Constants::CREATED ) )
			{
				$subtemplate = new HTML::Template( filename => "$templatesdir/largefile/menu_sub.tmpl" );
				$subtemplate->param( NAME => $obj->get_filename);
	            $subtemplate->param( HISTORY => gettext("History"));
	            $subtemplate->param( OID => $obj->get_oid );
			}
			else
			{
				$subtemplate = new HTML::Template( filename => "$templatesdir/largefile/menu_sub_wait.tmpl" );
				$subtemplate->param( NAME => $obj->get_filename );
	            $subtemplate->param( HISTORY => gettext("History"));
	            $subtemplate->param( REASON => Vhffs::Functions::status_string_from_status_id( $obj->get_status ));
	            $subtemplate->param( OID => $obj->get_oid );
			}


			$output .= $subtemplate->output if( defined $subtemplate );
			undef( $subtemplate );
		}
	}

	$template->param( SUB_LARGEFILE => $output );

	return $template->output;
}

sub create_largefile
{
	my $vhffs 		= shift;
	my $filename	= shift;
	my $size		= shift;
	my $type		= shift;
	my $licence		= shift;
	my $hash		= shift;
	my $description	= shift;
	my $user		= shift;
	my $group		= shift;

    return -1 if( $user->fetch < 0 );
    return -2 if( $group->fetch < 0 );

    my $file = new Vhffs::Services::LargeFile( $vhffs , $filename , $user , $group );

    return undef if( ! defined $file );

    $file->set_user( $user );
    $file->set_group( $group );
	return undef if( $file->fetch > 0 );
	return undef if( $file->set_licence( $licence ) < 0 );
    return undef if ( $file->set_size( $size ) < 0 );
    return undef if ( $file->set_type( $type ) < 0 );
    return undef if ( $file->set_hash( $hash ) < 0 );

	$file->set_status( Vhffs::Constants::WAITING_FOR_VALIDATION );
	$file->set_description( $description );

    return undef if( $file->create < 0 );

    return undef if ( Vhffs::Acl::add_acl( $user , $file , Vhffs::Constants::ACL_DELETE , $vhffs ) < 0 );
    return undef if ( Vhffs::Acl::add_acl( $group , $file , Vhffs::Constants::ACL_VIEW , $vhffs ) < 0 );

	return undef if( $file->fetch < 0 );
	return undef if( $file->set_licence( $licence ) < 0 );
	if( defined $hash )
	{
		$file->set_hash( $hash );
	}
	return undef if( $file->commit < 0 );

    return $file;
}



1;
