#!/usr/bin/perl
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Vhffs::Robots::Cvs;

use Vhffs::Services::Cvs;
use Vhffs::Constants;
use Vhffs::Functions;


sub getall_refused
{
	my( $main ) = @_;
	
	my $query = "SELECT c.cvsroot, c.object_id FROM  vhffs_cvs c, vhffs_object o WHERE o.state='".Vhffs::Constants::VALIDATION_REFUSED."' AND c.object_id = o.object_id";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('cvsroot') );
}



sub get_all_cvs_to_create
{
	my( $main ) = @_;
	
	my $query = "SELECT c.cvsroot, c.object_id FROM  vhffs_cvs c, vhffs_object o WHERE o.state='".Vhffs::Constants::WAITING_FOR_CREATION."' AND c.object_id = o.object_id";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('cvsroot') );
}

sub create_cvs
{
	my $cvs = shift;

	if( $cvs->fetch > 0 )
	{
		if( create_repository( $cvs ) > 0 )
		{
			$cvs->set_status( Vhffs::Constants::ACTIVATED );
			$cvs->commit;
			$cvs->add_history("Cvsroot now created !" );
		}
		else
		{
			$cvs->add_history("Cvsroot NOT created, bots encounters errors !" );
		}
	}
}


sub delete_cvs
{
	my $cvs = shift;

	if( $cvs->fetch > 0 )
	{
		my $cvsroot = $cvs->get_cvsroot;
		my $vhffs = $cvs->{'main'};

		my $dir = $cvs->get_dir;
		my $cmd = "rm -rf $dir";
		system( $cmd );
		$cvs->delete;
	}
}



sub create_repository
{
	my $cvs = shift;
	return if( $cvs->fetch < 0 );
	my $cvsroot = $cvs->get_cvsroot;
	my $vhffs = $cvs->{'main'};

	my $datadir = $vhffs->get_config->get_datadir;
	
	my $dir = $cvs->get_dir;
	return -1 if( $dir =~ /\./ );

	Vhffs::Functions::create_dir( $dir ) if( ! -d $dir );

	if( -x "/usr/bin/cvs" )
	{
		$command = "/usr/bin/cvs -d $dir init";
		system( $command );

		if( Vhffs::Functions::change_owner_recur( $dir , $cvs->get_owneruid , $cvs->get_ownergid ) < 0 )
		{
			return -2;
		}
		else
		{
			return 1;
		}
			
	}
	else
	{
		return -1;
	}
}

sub getall_cvs_to_delete
{
	my( $main ) = @_;
	
	my $query = "SELECT c.cvsroot, c.object_id FROM  vhffs_cvs c, vhffs_object o WHERE o.state='".Vhffs::Constants::TO_DELETE."' AND c.object_id = o.object_id";

	$request = $main->{'db'}->{'DB_READ'}->prepare( $query ) or return -2;
	return undef if ( $request->execute() <= 0 );

	return( $request->fetchall_hashref('cvsroot') );
}

1;

