/*
# Copyright (c) vhffs project and its contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
#
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
#2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
#3. Neither the name of vhffs nor the names of its contributors 
#   may be used to endorse or promote products derived from this 
#   software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
#FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
#COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
#INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
#CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
*/	

CREATE TABLE vhffs_object
(
	object_id serial,
	owner_uid int4 NOT NULL,
	date_creation timestamp,
	state varchar NOT NULL,
	description TEXT,
	CONSTRAINT vhffs_object_pkey PRIMARY KEY (object_id)
) WITH OIDS;


CREATE TABLE vhffs_acl
(
	acl_id serial,
	oid_src int4,
	perm int4 NOT NULL,
	oid_dst int4 ,
	CONSTRAINT vhffs_acl_pkey PRIMARY KEY (acl_id)
) WITH OIDS;

CREATE TABLE vhffs_groups
(
	gid serial,
	groupname varchar(50) NOT NULL DEFAULT ' ',
	passwd varchar(20),
	quota int4 NOT NULL,
	quota_used int4 NOT NULL DEFAULT 0,
	object_id int4 NOT NULL,
	CONSTRAINT vhffs_groups_pkey PRIMARY KEY (gid)
) WITH OIDS;

CREATE TABLE vhffs_users
(
	uid serial,
	gid int4 NOT NULL,
	username varchar(50) NOT NULL DEFAULT ' ',
	shell varchar(20) NOT NULL DEFAULT ' ',
	passwd varchar(40) NOT NULL DEFAULT ' ',
	homedir varchar(40) NOT NULL DEFAULT ' ',
	admin int4 NOT NULL,
	object_id int4 NOT NULL,
	CONSTRAINT vhffs_users_pkey PRIMARY KEY (uid)
) WITH OIDS;

CREATE TABLE vhffs_boxes
(
	domain varchar,
	local_part varchar,
	domain_hash varchar NOT NULL,
	password_hash varchar NOT NULL,
	mbox_name varchar NOT NULL,
	password varchar NOT NULL,
	nospam boolean,
	novirus boolean,
	CONSTRAINT vhffs_boxes_pkey PRIMARY KEY (domain,local_part)
) WITH OIDS;


CREATE TABLE vhffs_confirmation
(
	cid SERIAL,
	code varchar(10),
	CONSTRAINT vhffs_confirmation_pkey PRIMARY KEY (cid)
) WITH OIDS;




CREATE TABLE vhffs_cvs
(
	cvs_id serial,
	cvsroot varchar NOT NULL,
	owner_uid int4, 
	owner_gid int4,
	public boolean NOT NULL,
	object_id int4,
	CONSTRAINT vhffs_cvs_pkey PRIMARY KEY (cvs_id)
) WITH OIDS;

CREATE TABLE vhffs_dns_global
(
	dns_id serial,
	domain varchar(255) NOT NULL,
	owner_gid int4,
	owner_uid int4,
	object_id int4,
	CONSTRAINT vhffs_dns_pkey4 PRIMARY KEY ( dns_id )
) WITH OIDS;

CREATE TABLE vhffs_dns_rr (
	id serial,
	zone INTEGER NOT NULL,
	name varchar(64) NOT NULL,
	type VARCHAR(5) NOT NULL,
	data varchar(128) NOT NULL,
	aux INTEGER NOT NULL DEFAULT 0,
	ttl INTEGER NOT NULL DEFAULT 86400,
	CONSTRAINT vhffs_dns_rr_pkey PRIMARY KEY (id)
) WITH OIDS;	

CREATE TABLE vhffs_dns_soa
(
	id serial,
	origin varchar(255) NOT NULL,
	ns varchar(255) NOT NULL,
	mbox varchar(255) NOT NULL,
	serial INTEGER NOT NULL DEFAULT 1,
	refresh INTEGER NOT NULL DEFAULT 28800,
	retry INTEGER NOT NULL DEFAULT 7200,
	expire INTEGER NOT NULL DEFAULT 604800,
	minimum INTEGER NOT NULL DEFAULT 86400,
	ttl INTEGER NOT NULL DEFAULT 86400,
	active INTEGER,
	CONSTRAINT vhffs_dns_soa_pkey PRIMARY KEY (id)
) WITH OIDS;

CREATE TABLE vhffs_forward
(
	domain varchar NOT NULL,
	local_part varchar NOT NULL,
	remote_name varchar NOT NULL,
	password varchar NOT NULL,
	CONSTRAINT vhffs_forward_pkey PRIMARY KEY (domain , local_part)
) WITH OIDS;

CREATE TABLE vhffs_groups_info
(
	gid int4 NOT NULL,
	owner_uid int4,
	uid_mod int4,
	CONSTRAINT vhffs_groups_info_pkey PRIMARY KEY (gid)
) WITH OIDS;

CREATE TABLE vhffs_httpd
(
	httpd_id serial,
	servername varchar(200) NOT NULL DEFAULT ' ',
	crawl int4 NOT NULL default '1',
	owner_http int4 NOT NULL,
	owner_rev int4 NOT NULL,
	owner_uid int4,
	owner_gid int4,
	trafic int4 NOT NULL,
	alert_state int4 NOT NULL default '0',
	alert_limit int4 NOT NULL default '80',
	object_id int4 NOT NULL,
	CONSTRAINT vhffs_httpd_pkey PRIMARY KEY (httpd_id)
) WITH OIDS;

CREATE TABLE vhffs_largefile
(
	file_id SERIAL,
	filename varchar NOT NULL,
	size			int4 NOT NULL,
	type			varchar NOT NULL,
	licence			varchar,
	hash			varchar,
	creating_date	DATE,
	last_check		DATE,
	end_date		DATE,
	username		varchar,
	password		varchar,
	home			varchar,
	ncheck			int4 DEFAULT '0',
	virtual_uid		int4,
	virtual_gid		int4,
	owner_uid 		int4, 
	owner_gid		int4,
	object_id		int4,
	CONSTRAINT vhffs_largefile_pkey PRIMARY KEY( filename , owner_gid)
) WITH OIDS;

CREATE TABLE vhffs_repository
(
	repository_id serial,
	name varchar NOT NULL,
	owner_uid int4 NOT NULL,
	owner_gid int4 NOT NULL,
	quota int4 NOT NULL,
	quota_used int4 NOT NULL DEFAULT 0,
	object_id int4 NOT NULL,
	CONSTRAINT vhffs_repository_pkey PRIMARY KEY (repository_id)
) WITH OIDS;

CREATE TABLE vhffs_mailings
(
	id_mailing serial,
	subject VARCHAR NOT NULL,
	message TEXT NOT NULL,
	date timestamp,
	state int4 NOT NULL,
	CONSTRAINT vhffs_mailings_pkey PRIMARY KEY (id_mailing)
) WITH OIDS;


CREATE TABLE vhffs_mxdomain
(
	mxdomain_id serial,
	domain varchar UNIQUE,
	unix_user varchar NOT NULL,
	boxes_path varchar NOT NULL,
	max_popbox int4 NOT NULL,
	catchall varchar NOT NULL,
	owner_uid int4 NOT NULL,
	owner_gid int4 NOT NULL,
	object_id int4 NOT NULL,
	CONSTRAINT vhffs_mxdomain_pkey PRIMARY KEY (mxdomain_id)
) WITH OIDS;

CREATE TABLE vhffs_ml
(
	ml_id serial,
	local_part VARCHAR NOT NULL,
	domain VARCHAR REFERENCES vhffs_mxdomain( domain ),
	prefix VARCHAR ,
	owner_uid int4, 
	owner_gid int4,
	object_id int4,
	CONSTRAINT vhffs_ml_pkey PRIMARY KEY (ml_id)
) WITH OIDS;

CREATE TABLE vhffs_ml_prefs
(
	ml_id int4 REFERENCES vhffs_ml( ml_id ) PRIMARY KEY,
	admin VARCHAR NOT NULL,
	open_post int4 NOT NULL,
	open_archive int4 NOT NULL,
	open_sub int4 NOT NULL,
	reply_to int4,
	moderated int4 NOT NULL,
	url VARCHAR
) WITH OIDS;

CREATE TABLE vhffs_ml_subscribers
(
	sub_id serial,
	member VARCHAR NOT NULL,
	perm int4 NOT NULL,
	active int4 NOT NULL,
	hash varchar NOT NULL,
	ml_id int4 NOT NULL,
	CONSTRAINT vhffs_ml_subscribers_pkey PRIMARY KEY (sub_id)
) WITH OIDS;

CREATE TABLE vhffs_ml_lang
(
/* TODO use an int4 and link it to vhffs_ml_subscribers(sub_id) */
	member VARCHAR NOT NULL,
	lang   VARCHAR NOT NULL,
	CONSTRAINT vhffs_ml_lang_pkey PRIMARY KEY (member)
) WITH OIDS;

CREATE TABLE vhffs_mysql
(
	mysql_id serial,
	dbname varchar(200) NOT NULL,
	dbuser varchar(200) NOT NULL,
	owner_uid int4, 
	owner_gid int4,
	dbpass VARCHAR(32) NOT NULL,
	object_id int4,
	CONSTRAINT vhffs_mysql_pkey PRIMARY KEY (mysql_id)
) WITH OIDS;

CREATE TABLE vhffs_notes
(
	object_id int4 NOT NULL,
	note int4 DEFAULT '0',
	CONSTRAINT vhffs_notes_pkey PRIMARY KEY(object_id)
) WITH OIDS;

CREATE TABLE vhffs_history
(
	history_id serial,
	object_id int4,
	date timestamp,
	message varchar(200),
	CONSTRAINT vhffs_history_pkey PRIMARY KEY (history_id)
) WITH OIDS;

CREATE TABLE vhffs_panel_user_prefs
(
	uid int4,
	language varchar,
	theme varchar,
	CONSTRAINT vhffs_panel_user_prefs_pkey PRIMARY KEY (uid)
) WITH OIDS;

CREATE TABLE vhffs_pgsql
(
	pgsql_id serial,
	dbname varchar(200) NOT NULL,
	dbuser VARCHAR(16) NOT NULL,
	dbpass VARCHAR(32) NOT NULL,
	owner_uid int4, 
	owner_gid int4,
	object_id int4,
	CONSTRAINT vhffs_pgsql_pkey PRIMARY KEY (pgsql_id)
) WITH OIDS;

CREATE TABLE vhffs_svn
(
	svn_id SERIAL,
	reponame varchar NOT NULL,
	owner_uid int4, 
	owner_gid int4,
	public int4 NOT NULL,
	object_id int4,
	CONSTRAINT vhffs_svn_pkey PRIMARY KEY( reponame , owner_gid)
) WITH OIDS;

CREATE TABLE vhffs_user_group
(
	uid int4 ,
	gid int4 ,
	state int4 ,
	CONSTRAINT vhffs_user_group_pkey PRIMARY KEY (uid,gid)
)WITH OIDS;

CREATE TABLE vhffs_user_info
(
	uid int4 NOT NULL,
	date_creation timestamp,
	firstname varchar(250),
	lastname varchar(250),
	address text,
	zipcode int4,
	city varchar(250),
	country varchar(250),
	mail varchar(200),
	gpg_key varchar(250),
	CONSTRAINT vhffs_user_info_pkey PRIMARY KEY (uid)
) WITH OIDS;


/****** Non primary key constraints.
      Defining foreign keys here allow to create tables in any order.
*******/

ALTER TABLE vhffs_acl ADD CONSTRAINT fk_vhffs_acl_vhffs_object_dst FOREIGN KEY (oid_dst) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;
ALTER TABLE vhffs_acl ADD CONSTRAINT fk_vhffs_acl_vhffs_object_src FOREIGN KEY (oid_src) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_groups ADD CONSTRAINT fk_vhffs_group_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_users ADD CONSTRAINT fk_vhffs_users_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object (object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_cvs ADD CONSTRAINT fk_vhffs_cvs_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_cvs ADD CONSTRAINT fk_vhffs_cvs_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_cvs ADD CONSTRAINT fk_vhffs_cvs_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_dns_global ADD CONSTRAINT fk_vhffs_dns_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_dns_global ADD CONSTRAINT fk_vhffs_dns_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_dns_global ADD CONSTRAINT fk_vhffs_dns_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_dns_rr ADD CONSTRAINT fk_vhffs_dns_rr_vhffs_dns FOREIGN KEY (zone) REFERENCES vhffs_dns_global(dns_id) ON DELETE CASCADE;
ALTER TABLE vhffs_dns_rr ADD CONSTRAINT fk_vhffs_dns_rr_chk_type CHECK (type='A' OR type='AAAA' OR type='CNAME' OR type='HINFO' OR type='MX' OR type='NS' OR type='PTR' OR type='RP' OR type='SRV' OR type='TXT');

ALTER TABLE vhffs_groups_info ADD CONSTRAINT fk_vhffs_groups_info_vhffs_groups FOREIGN KEY (gid) REFERENCES vhffs_groups (gid);
ALTER TABLE vhffs_groups_info ADD CONSTRAINT fk_vhffs_groups_info_vhffs_users FOREIGN KEY (uid_mod) REFERENCES vhffs_users (uid);

ALTER TABLE vhffs_httpd ADD CONSTRAINT fk_vhffs_httpd_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_httpd ADD CONSTRAINT fk_vhffs_httpd_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);

ALTER TABLE vhffs_largefile ADD CONSTRAINT fk_vhffs_largefile_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_largefile ADD CONSTRAINT fk_vhffs_largefile_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_largefile ADD CONSTRAINT fk_vhffs_largefile_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_repository ADD CONSTRAINT fk_vhffs_vhffs_repository_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_repository ADD CONSTRAINT fk_vhffs_vhffs_repository_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_repository ADD CONSTRAINT fk_vhffs_vhffs_repository_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_mxdomain ADD CONSTRAINT fk_vhffs_mxdomain_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_mxdomain ADD CONSTRAINT fk_vhffs_mxdomain_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_mxdomain ADD CONSTRAINT fk_vhffs_mxdomain_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_ml ADD CONSTRAINT fk_vhffs_ml_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_ml ADD CONSTRAINT fk_vhffs_ml_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_ml ADD CONSTRAINT fk_vhffs_ml_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_ml_subscribers ADD CONSTRAINT fk_vhffs_ml_subscribers_vhffs_ml FOREIGN KEY (ml_id) REFERENCES vhffs_ml(ml_id);

ALTER TABLE vhffs_mysql ADD CONSTRAINT fk_vhffs_mysql_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_mysql ADD CONSTRAINT fk_vhffs_mysql_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_mysql ADD CONSTRAINT fk_vhffs_mysql_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_notes ADD CONSTRAINT fk_vhffs_notes FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_history ADD CONSTRAINT fk_vhffs_history_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_user_info ADD CONSTRAINT fk_vhffs_user_info_vhffs_users FOREIGN KEY (uid) REFERENCES vhffs_users (uid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE vhffs_pgsql ADD CONSTRAINT fk_vhffs_pgsql_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_pgsql ADD CONSTRAINT fk_vhffs_pgsql_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_pgsql ADD CONSTRAINT fk_vhffs_pgsql_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_svn ADD CONSTRAINT fk_vhffs_svn_vhffs_users FOREIGN KEY (owner_uid) REFERENCES vhffs_users(uid);
ALTER TABLE vhffs_svn ADD CONSTRAINT fk_vhffs_svn_vhffs_groups FOREIGN KEY (owner_gid) REFERENCES vhffs_groups(gid);
ALTER TABLE vhffs_svn ADD CONSTRAINT fk_vhffs_svn_vhffs_object FOREIGN KEY (object_id) REFERENCES vhffs_object(object_id) ON DELETE CASCADE;

ALTER TABLE vhffs_user_group ADD CONSTRAINT fk_vhffs_user_group_vhffs_users FOREIGN KEY (uid) REFERENCES vhffs_users (uid) ON UPDATE CASCADE ON DELETE CASCADE;
ALTER TABLE vhffs_user_group ADD CONSTRAINT fk_vhffs_user_group_vhffs_groups FOREIGN KEY (gid) REFERENCES vhffs_groups (gid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE vhffs_panel_user_prefs ADD CONSTRAINT fk_vhffs_panel_user_prefs_vhffs_users FOREIGN KEY (uid) REFERENCES vhffs_users(uid) ON DELETE CASCADE;


CREATE VIEW vhffs_passwd AS
SELECT uid, gid, username, shell, '*'::character varying AS passwd, homedir
FROM vhffs_users;

CREATE VIEW vhffs_shadow AS
SELECT uid, gid, username, shell, passwd, '0'::int4 as newtok , '0'::int4 as expired , homedir
FROM vhffs_users;

CREATE VIEW vhffs_forum AS
SELECT users.username, users.passwd, user_info.firstname, user_info.lastname, user_info.mail, user_info.date_creation, object.state
FROM vhffs_users users, vhffs_user_info user_info, vhffs_object object
WHERE user_info.uid=users.uid AND object.object_id=users.object_id;
