#! /usr/bin/perl

# modobot is an IRC bot which allow you to validate
# VHFFS objects through IRC
# Written by Florent Bayle

use POSIX qw(locale_h);
use locale;
use strict;
use warnings;
use Locale::gettext;
use Data::Dumper;

use Vhffs::User;
use Vhffs::Group;
use Vhffs::Main;
use Vhffs::Services::Httpd;
use Vhffs::Acl;
use Vhffs::Constants;
use Vhffs::Note;

use Net::IRC;
use Text::Wrapper;

use constant
{
	SERVER	=> "PUT YOUR IRC SERVER HERE",
	PORT	=> "6667",
	CHANNEL => "IRC CHANNEL TO JOIN",
	NAME	=> "BOT NAME",
};


################# DO NOT EDIT BELOW THIS LINE ######################

# Configuration
my $srv = SERVER;
my $chan = CHANNEL;
my $botname = NAME;
my $bot;
my $port = PORT;

my @oldgroup = ();
my @oldweb = ();
my @olddns = ();
my @oldcvs = ();
my @oldmailing = ();
my @oldsvn = ();
my @oldmail = ();
my @oldmysql = ();
my @oldpostgres = ();
my @oldrepository = ();

my $irc=new Net::IRC;

# Connection
my $conn=$irc->newconn(Nick     =>  $botname,
                       Server   =>      $srv,
                       Port     =>     $port,
                       Username =>  "pacman",
                       Ircname  =>  "Bernard Minet");

my $vhffs = init Vhffs::Main;



sub deletenl
{
    ($_) = @_;
    $_ =~ s/\n/ /g;
    $_ =~ s/\r//g;
    return $_;
} # deletenl


sub list_moderation
{
    #Treat Groups to moderate
    use Vhffs::Panel::Group;
    my $groups = Vhffs::Group::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );
    my $temp;

    if( defined $groups )
    {
GR:     foreach $temp ( @{$groups} )
        {
	    map { last GR if (($_->get_oid eq $temp->get_oid) && ($_->get_groupname eq $temp->get_groupname)); } @oldgroup;
            if( $vhffs->get_config->use_notation == 1 )
            {
                irc_msg ("Groupe :      " . $temp->get_oid . "   " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_groupname . "   " . Vhffs::Functions::stripslashes($temp->get_description));
            }
            else
            {
                irc_msg ("Groupe :      " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_groupname . "    " . Vhffs::Functions::stripslashes($temp->get_description));
            }
        }
        @oldgroup = @{$groups};
    }
    else
    {
        @oldgroup = ();
    }

    if( $vhffs->get_config->use_web == 1 )
    {
        #Treat Website to moderate
        use Vhffs::Services::Httpd;
        my $webs = Vhffs::Services::Httpd::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION);

        if( defined $webs )
        {
WE:         foreach $temp ( @{$webs} )
            {
	        map { last WE if (($_->get_oid eq $temp->get_oid) && ($_->get_servername eq $temp->get_servername)); } @oldweb;

                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("Web :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_servername . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("Web :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_servername . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
            }
            @oldweb = @{$webs};
        }
	else
	{
            @oldweb = ();	
	}
    }

    if( $vhffs->get_config->use_dns == 1 )
    {
        #Treat DNS to moderate
        use Vhffs::Services::DNS;
        my $nss = Vhffs::Services::DNS::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined $nss )
        {
DN:         foreach $temp ( @{$nss} )
            {
	        map { last DN if (($_->get_oid eq $temp->get_oid) && ($_->get_domain eq $temp->get_domain)); } @olddns;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("DNS :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_domain . "    " . Vhffs::Functions::stripslashes($temp->get_description));
		}
                else
                {
                    irc_msg ("DNS :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_domain . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }

            }
        @olddns = @{$nss};
        }
        else
        {
            @olddns = ();
        }
    }

    if( $vhffs->get_config->use_cvs == 1 )
    {
        #Treat CVS to moderate
        use Vhffs::Services::Cvs;
        my $repos = Vhffs::Services::Cvs::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined $repos )
        {
CV:         foreach $temp ( @{$repos} )
            {
	        map { last CV if (($_->get_oid eq $temp->get_oid) && ($_->get_cvsroot eq $temp->get_cvsroot)); } @oldcvs;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("CVS :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_cvsroot . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("CVS :          " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_cvsroot . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
            }
            @oldcvs = @{$repos};
        }
        else
        {
            @oldcvs = ();
        }
    }

    if( $vhffs->get_config->use_mailing == 1 )
    {
        #Treat Mailing lists to moderate
        use Vhffs::Panel::Mailinglist;
        my $mls = Vhffs::Services::Mailing::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined $mls )
        {
ML:         foreach $temp ( @{$mls} )
            {
                map { last ML if (($_->get_oid eq $temp->get_oid) && ($_->get_localpart eq $temp->get_localpart)); } @oldmailing;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("Mailing list : " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_localpart . "@" . $temp->get_domain . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("Mailing list : " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_localpart . "@" . $temp->get_domain . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
            }
            @oldmailing = @{$mls};
        }
        else
        {
            @oldmailing = ();
        }
    }

    if( $vhffs->get_config->use_svn == 1 )
    {
        #Treat Svn to moderate
        use Vhffs::Services::Svn;
        my $repos = Vhffs::Services::Svn::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined $repos )
        {
SV:         foreach $temp ( @{$repos} )
            {
                map { last SV if (($_->get_oid eq $temp->get_oid) && ($_->get_reponame eq $temp->get_reponame)); } @oldsvn;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("SVN :          " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_reponame . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("SVN :          " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_reponame . "    " . Vhffs::Functions::stripslashes($temp->get_description));
		}
            }
        @oldsvn = @{$repos};
        }
        else
        {
            @oldsvn = ();
        }
    }

    if( $vhffs->get_config->use_mail == 1 )
    {
        #Treat Mail domains to moderate
        use Vhffs::Panel::Mail;
        my $mails = Vhffs::Services::Mail::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined $mails )
        {
MA:         foreach $temp ( @{$mails} )
            {
                map { last MA if (($_->get_oid eq $temp->get_oid) && ($_->get_domain eq $temp->get_domain)); } @oldmail;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("Mail :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_domain . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("Mail :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_domain . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
            }
            @oldmail = @{$mails};
        }
        else
        {
            @oldmail = ();
        }
    }

    if( $vhffs->get_config->use_mysql == 1 )
    {
        #Treat Mysql to moderate
        use Vhffs::Services::Mysql;
        my $dbs = Vhffs::Services::Mysql::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined $dbs )
        {
MY:         foreach $temp ( @{$dbs} )
            {
                map { last MY if (($_->get_oid eq $temp->get_oid) && ($_->get_dbname eq $temp->get_dbname)); } @oldmysql;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("MySql :        " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_dbname . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("MySql :        " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_dbname . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
            }
            @oldmysql = @{$dbs};
        }
        else
        {
            @oldmysql = ();
        }
    }

    if( $vhffs->get_config->use_postgres == 1 )
    {
        #Treat Postgres to moderate
        use Vhffs::Services::Postgres;
        my $dbs = Vhffs::Services::Postgres::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined @{$dbs} )
        {
PO:         foreach $temp ( @{$dbs} )
            {
                map { last PO if (($_->get_oid eq $temp->get_oid) && ($_->get_dbname eq $temp->get_dbname)); } @oldpostgres;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("Postgres :     " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_dbname . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("Postgres :     " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_dbname . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
            }
	    @oldpostgres = @{$dbs};
        }
        else
        {
            @oldpostgres = ();
        }
    }

    if( $vhffs->get_config->use_repository == 1 )
    {
        #Treat Repository to moderate
        use Vhffs::Services::Repository;
        my $repos = Vhffs::Services::Repository::getall( $vhffs , Vhffs::Constants::WAITING_FOR_VALIDATION );

        if( defined $repos )
        {
RP:         foreach $temp ( @{$repos} )
            {
	        map { last RP if (($_->get_oid eq $temp->get_oid) && ($_->get_name eq $temp->get_name)); } @oldrepository;
                if( $vhffs->get_config->use_notation == 1 )
                {
                    irc_msg ("REPOSITORY :         " . $temp->get_oid . "    " . $temp->get_user->get_username . "(" .  Vhffs::Note::get_note( $vhffs , $temp->get_user ) . ") [".$temp->get_user->get_lang."]   " . $temp->get_group->get_groupname . "    " . $temp->get_name . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
                else
                {
                    irc_msg ("REPOSITORY :          " . $temp->get_oid . "    " . $temp->get_user->get_username . "    " . $temp->get_group->get_groupname . "    " . $temp->get_name . "    " . Vhffs::Functions::stripslashes($temp->get_description));
                }
            }
            @oldrepository = @{$repos};
        }
        else
        {
            @oldrepository = ();
        }
    }

} # list_moderation


sub moderate
{
    my $oid;
    my $status; # 0 = refuse , 1 = accept
    my $object;
    my $raison;

    $oid    =       shift;
    $status =       shift;

    $object = new Vhffs::Object( $vhffs , $oid , '51' );

    if( ! defined $object )
    {
        irc_msg ("Error : Object is null");
        return ( -1 );
    }
    elsif( $object->fetch < 0 )
    {
        irc_msg ("Error : Cannot fetch object");
        return ( -1 );
    }
    elsif( $object->get_status != Vhffs::Constants::WAITING_FOR_VALIDATION )
    {
        irc_msg ("Error : Object is not waiting for validation");
        return ( -2 );
    }
    else
    {
        if( $status == 1 )
        {
            $object->set_status( Vhffs::Constants::WAITING_FOR_CREATION );
            my $username = Vhffs::User::get_name_by_uid( $vhffs , $object->get_owner_uid );
            my $userbis = new Vhffs::User( $vhffs , $username , '401' );
            if( $userbis->fetch > 0 )
            {
                my $mail;
                $mail  = gettext("Your request on VHFFS was accepted\n");
                $mail .= gettext("Please wait while we are creating your object\n");
                $mail .= gettext("Cheers,")."\n";
                $mail .= gettext("The Moderator and Admin team");

                my $subject = gettext("Your request on VHFFS platform");
                Vhffs::Functions::send_mail( $vhffs->get_config->get_moderator_mail , $userbis->get_mail , $subject , $mail );

                if( $vhffs->get_config->use_notation == 1 )
                {
                    use Vhffs::Note;
                    Vhffs::Note::inc_note( $vhffs , $userbis , 1 );
                }
            }
        }
        else
        {
	    $raison = shift;
            $object->set_status( Vhffs::Constants::VALIDATION_REFUSED );
	    $object->set_description( $raison );
            my $username = Vhffs::User::get_name_by_uid( $vhffs , $object->get_owner_uid );
            my $userbis = new Vhffs::User( $vhffs , $username , '401' );
            if( $userbis->fetch > 0 )
            {
       	        if( $vhffs->get_config->use_notation == 1 )
                {
                    use Vhffs::Note;
                    Vhffs::Note::inc_note( $vhffs , $userbis , -1 );
                }
            }
        }

        if( $object->commit < 0 )
        {
            irc_msg ("Error while commiting changes");
            return ( -3 );
        }

    }
    return 0;
}



sub on_ping {
    my ($self, $event) = @_;
    my $nick = $event->nick;

    $self->ctcp_reply($nick, join (' ', ($event->args)));
    print "[ping-from]  : {$nick}\n";
} # on_ping

sub on_ping_reply {
    my ($self, $event) = @_;
    my ($args) = ($event->args)[1];
    my ($nick) = $event->nick;

    $args = time - $args;
    print "[ping-rsp]  : from $nick: $args sec.\n";
} # on_ping_reply

sub on_cversion {
    my ($self, $event) = @_;
    my ($nick, $mynick) = ($event->nick, $self->nick);
    my $reply = "Vhffs Bot";
    print "[ctcp-version] : <$nick>\n";
    $self->ctcp_reply($nick, join(' ', ($event->args), $reply));
} # on_cversion

sub on_connect {
    my $self=shift;
    $bot=$self;
    $self->join($chan);
    irc_msg ("--> $botname started");
} # on_connect

sub is_modo
{
    my $user = new Vhffs::User( $vhffs, shift, 401 );
    my $ret;
    $ret = $user->fetch;
    if ( $ret >= 0 )
    {
        if( ($user->is_moderator == 1 ) || ( $user->is_admin ) == 1 )
        {
            return 1;
        }
    }
    return 0;
} # is_modo

sub get_desc
{
    my $name = shift;
    my $group;
    if ((! defined ($group= new Vhffs::Group( $vhffs , $name , '401' ) ) ) || ( $group->fetch < 0 ))
    {
        irc_msg ("$name : No such group");
    }
    else
    {
        irc_msg ("$name : " . Vhffs::Functions::stripslashes ($group->get_description));
    }
}

sub find_group_from_web
{
    my $name = shift;
    my $web;
    if ((! defined ($web = new Vhffs::Services::Httpd( $vhffs , $name, $name ) ) ) || ( $web->fetch < 0 ))
   {
       irc_msg ("$name : No such website");
   }
   else
   {
       irc_msg ("$name => group  ".Vhffs::Group::get_name_by_gid($vhffs,$web->get_ownergid));
   }

}

sub owner_info
{
  my $groupname = shift;
  my $group;
  if ((! defined ($group = new Vhffs::Group( $vhffs , $groupname,'401' ) ) ) || ( $group->fetch < 0 ))
  {
    irc_msg ("$groupname : No such groupname");
  }
  else
  {
    my $user = new Vhffs::User($vhffs,$group->get_owner_username,401);
    my $ret;
    $ret = $user->fetch;
    if ( $ret >= 0 )
    {
      irc_msg ($groupname." owned by ". $user->get_username ." (". $user->get_firstname ." ". $user->get_lastname ." [". $user->get_mail ."] )");
    }
    else
    {
     irc_msg ($groupname." : error fetching user");}
    }
}

sub fetch_usergroup
{
 my $groupname = shift;
 my $group;
 if ((! defined ($group= new Vhffs::Group( $vhffs , $groupname , '401' ) ) ) || ( $group->fetch < 0 ))
 {
  irc_msg ("$groupname : No such group");
 }
 else
 {
  my $users = Vhffs::Group::get_users($group);
  my $temp;
  foreach $temp ( @{$users} )
  {
   if ( $temp->get_username =~ $group->get_owner_username)
   {
    irc_msg(" @ ".$temp->get_username);
   }
   else
   {
    irc_msg(" + ".$temp->get_username);
   }
  }
 }
}

sub irc_msg
{
    my $text = shift;
    $text = deletenl ($text);
    my $wrapper = Text::Wrapper->new(columns => 300);
    my @text = split (/\n/, $wrapper->wrap($text));
    for $a (@text)
    {
        $bot->privmsg($chan, $a);
    }
}

sub on_public {
    my ($self, $event)=@_;
    my ($nick, $mynick)=($event->nick, $self->nick);
    my $texte=$event->{args}[0];

    if ($texte =~ m/^${mynick}: accept [0-9]+$/)
    {
        if (is_modo ($nick) == 1)
	{
	    $texte =~ s/^${mynick}: accept //;
            if (moderate( $texte , 1 ) == 0)
            {
                irc_msg("Object " . $texte . " accepted");
            }
	}
    }
    elsif ($texte =~ m/^${mynick}: refuse [0-9]+ .*$/)
    {
        if (is_modo ($nick) == 1)
	{
	    my $oid = $texte;
	    my $raison = $texte;
	    $raison =~ s/^${mynick}: refuse [0-9]+ //;
	    $oid =~ s/^${mynick}: refuse ([0-9]+) .*$/$1/;
            if (moderate( $oid , 0 , $raison) == 0)
            {
                irc_msg("Object " . $oid . " refused");
            }
	}
    }
    elsif ($texte =~ m/^${mynick}: help$/)
    {
        irc_msg("Commands :");
        irc_msg("help - show this help");
        irc_msg("accept <oid> - accept object with id <oid>");
        irc_msg("refuse <oid> <reason> - refuse object with id <oid> for reason <reason>");
	irc_msg("list - force listing of all objects waiting for moderation");
	irc_msg("desc <group> - give the description of <group>");
        irc_msg("web2group <website> - give the groupe name of <website>");
        irc_msg("owner <group> - give owner information of <group>");
        irc_msg("lsgroup <group> - give the list of users of <group>"); 

   }
    elsif ($texte =~ m/^${mynick}: list$/)
    {
        @oldgroup = ();
        @oldweb = ();
        @olddns = ();
        @oldcvs = ();
        @oldmailing = ();
        @oldsvn = ();
        @oldmail = ();
        @oldmysql = ();
        @oldpostgres = ();
	@oldrepository = ();
		list_moderation();
    }
    elsif ($texte =~ m/^${mynick}: desc [a-z0-9]+$/)
    {
        my $groupid = $texte;
	$groupid =~ s/^${mynick}: desc //;
        get_desc ($groupid);
    }

    elsif ($texte =~ m/^${mynick}: web2group [a-z0-9\.\-]+$/)
    {
        my $webtogroup = $texte;
        $webtogroup =~ s/^${mynick}: web2group //;
        find_group_from_web($webtogroup);
    }

    elsif ($texte =~ m/^${mynick}: owner [a-z0-9]+$/)
    {
        my $groupid = $texte;
        $groupid =~ s/^${mynick}: owner //;
        owner_info ($groupid);
    }

    elsif ($texte =~ m/^${mynick}: lsgroup [a-z0-9]+$/)
    {
        my $groupid = $texte;
        $groupid =~ s/^${mynick}: lsgroup //;
        fetch_usergroup ($groupid);
    }

} # on_public

sub on_kick {
    my $self=shift;
    $self->join($chan);
} # on_kick

$conn->add_handler        ('cping',    \&on_ping);
$conn->add_handler        ('crping',   \&on_ping_reply);
$conn->add_global_handler ('376',      \&on_connect);
$conn->add_handler        ('cversion', \&on_cversion);
$conn->add_handler        ('public',   \&on_public);
$conn->add_handler        ('kick',     \&on_kick);

sub CatchAlrm
{
    list_moderation ();
    alarm 60;
}


local $SIG{ALRM} = \&CatchAlrm;
alarm 5;

$irc->start;

